<?php
/*
Plugin Name: Facebook Status Updater
Plugin URI: http://www.francesco-castaldo.com/plugins-and-widgets/fb-status-updater/
Description: For each new post published, the plugin updates the status of the given Facebook profile or page. <a href="/wp-admin/options-general.php?page=fb-status-updater.php">Settings</a> | <a href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=2797439" target="_blank">Donate</a>
Author: Francesco Castaldo
Version: 1.1
Author URI: http://www.francesco-castaldo.com/


Installing
1. Unzip and upload 'fb-status-updater' directory to your '/wp-content/plugins/' directory
2. Activate the plugin through the 'Plugins' menu in the Admin panel
3. Fill required data in the plugin options page 

Changelog
1.1 = log email format issue solved, added one more wall to post to, send one post just one time
1.0 = First public release.

*/

/*
Copyright 2009  Francesco Castaldo  (email : fcastaldo@gmail.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

$fbStatusUpdatePath = __FILE__;
$fbStatusUpdatePath = substr($fbStatusUpdatePath, 0,  strrpos($fbStatusUpdatePath, "/"));
$fbStatusCookieFile = $fbStatusUpdatePath."/fbSessionData.txt";

function getMicrotime() {
	$tmp = split(" ", microtime());
	$rt = $tmp[0] + $tmp[1];
	return $rt;
}

function sendLogEmail($address, $message) {
	$subject = "[Fb Status Updater] Status Update!";
	$emailBody = "Hey,\n";
	$emailBody .= "a new post have been published from your blog and the Fb Status Updater plugin attempted to share it on Facebook.\n";
	$emailBody .= $message;
	$emailBody .= "\nTo disable these email, just go to the plugin settings page, delete the \"Log email\" field and save.\n\nCheers!";
	wp_mail($address, $subject, $emailBody);
}

function deleteFbSessionData() {
	global $fbStatusCookieFile;
	// if the file is not writable, the function is not called so there's no need for double check also inside the function
	if ($handle = fopen($fbStatusCookieFile, 'w')) {
		if (fwrite($handle, "") === FALSE) {
			return false;
		}
	} else {
		fclose($handle);
		return false;
	}
	fclose($handle);
	return true;
}

function getPage($url, $postData = null) {
	global $fbStatusUpdatePath, $fbStatusCookieFile;

	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $url);
	curl_setopt($ch, CURLOPT_HEADER, false);
	curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
	curl_setopt($ch, CURLOPT_COOKIEJAR, $fbStatusCookieFile);
	curl_setopt($ch, CURLOPT_COOKIEFILE, $fbStatusCookieFile);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_REFERER, $url);
	curl_setopt($ch, CURLOPT_USERAGENT, "Mozilla/5.0 (Windows; U; Windows NT 5.1; it; rv:1.9.0.6; .NET CLR 3.0; ffco7) Gecko/2009011913 Firefox/3.0.6");

	if ($postData != null) {
		curl_setopt($ch, CURLOPT_POSTFIELDS, $postData);
		curl_setopt($ch, CURLOPT_POST, true);
	}
	$response = curl_exec($ch);

	unset($ch);

	return $response;
}

function fbStatusUpdater($post_ID) {
	
	global $fbStatusCookieFile;
	$startTime = getMicrotime();

	$statusUpdate = get_option('fb-status-update');

	if ($statusUpdate === false) {
		// no settings stored
		return;
	} else {

		if (isSet($statusUpdate["fb-post-ids"]) && $statusUpdate["fb-post-ids"] != "") {
			// check if the current post id has already been sent to facebook
			if (strpos($statusUpdate["fb-post-ids"], "#".$post_ID."#") !== false) {
				// the post was already sent to facebook, do not lose time here
				sendLogEmail($address, "The post ".$post_ID." was already sent to Facebook so I skipped the whole process");
				return;
			}
		} else {
			//inizialize it so that every post id is surrounded by # Example:
			// #1#2#3#100#1850#
			$statusUpdate["fb-post-ids"] = "#";
		}


		$fbUser = null;
		$fbPassword = null;
		$fbWallId = null;
		$fbWallId2 = null;
		$fbLogEmail = null;
		$fbLogMessage = "";

		if (isSet($statusUpdate["fb-email"]) && isSet($statusUpdate["fb-password"])) {
			$fbUser = $statusUpdate["fb-email"];
			$fbPassword = $statusUpdate["fb-password"];
		} else {
			// no settings stored
			$statusUpdate = false;
			return;
		}
		if (isSet($statusUpdate["fb-wall-id"])) {
			$fbWallId = $statusUpdate["fb-wall-id"];
		}
		if (isSet($statusUpdate["fb-wall-id-2"])) {
			$fbWallId2 = $statusUpdate["fb-wall-id-2"];
		}
		if (isSet($statusUpdate["fb-log-email"])) {
			$fbLogEmail = $statusUpdate["fb-log-email"];
		}
	}

	if (!is_writable($fbStatusCookieFile)) {
		echo("Your post has been published but there was an error while updating the Facebook status. The file <strong>".$fbStatusCookieFile."</strong> is not writable from PHP. Please ensure PHP has the correct permissions set to write and update that file. If you don't know what I'm speaking about, please contact your server admin / webmaster. If you don't want to see this message every time you publish a new post while you try solving the problem, just <a href=\"/wp-admin/plugins.php\">disable this plugin</a>. <a href=\"http://codex.wordpress.org/Changing_File_Permissions\">More about file permissions on Wordpress</a>");
		exit();
	}

	$post = get_post($post_ID);
	//$slug = $post->post_name;
	$postUrl = get_permalink($post_ID);
	$title = $post->post_title;
	$postStatus = $post->post_status;
	$postDate = $post->post_date;
	$postModified = $post->post_modified;
	unset($post);

	// controllo se la pubblicazione avverr in futuro. In quel caso non si fa nulla
	if ($postStatus == "future") {
		if ($fbLogEmail != null) {
			sendLogEmail($fbLogEmail, "The artile \"".$title."\" has not been shared on Facebook because its publication is due to happen in the future.\n");
		}
		return;
	}

	
	// Ulr shorteners seem to be not that appreciated from (dorky) Facebook: it asks for captcha.
	/*
	$isGdApiUrl = "http://is.gd/api.php?longurl=".$postUrl;
	$shortUrl = getPage($isGdApiUrl);

	if (strpos($shortUrl, "Error: ") === false) {
		// it didn't work, use the full url
		$postUrl = $shortUrl;
	}
	*/

	$title .= " ".$postUrl;

	$loginUrl = "https://login.facebook.com/login.php?m&locale=en_US&next=http://m.facebook.com/home.php%3Flocale=en_US";
	$postData = "locale=en_US&email=".$fbUser."&pass=".$fbPassword."&persistent=1&login=".urlencode("Log In");

	// clean the session data before starting
	// the file should already be clean but... you never know if someone else touches it with some malicious plugin
	deleteFbSessionData();

	$loginResponse = getPage($loginUrl, $postData);

	if (strpos($loginResponse, "type=\"password\"") !== false) {
		// login fallito
		echo("Your post has been published but Facebook login failed. <a href=\"/wp-admin/edit.php\">Go to your posts</a> or <a href=\"/wp-admin/options-general.php?page=fb-status-updater.php\">check your Facebook login data</a>");
		deleteFbSessionData();
		exit();
	}

	// status update
	preg_match('/name="post_form_id" value="(.*?)"/i', $loginResponse, $postFormId);

	if (isSet($postFormId[1])) {

		$postData = "post_form_id=".$postFormId[1]."&status=".urlencode($title)."&update=Update+Status&locale=en_US";
		$statusFormUrl = "http://m.facebook.com/a/home.php?locale=en_US";

		$updateResponse = getPage($statusFormUrl, $postData);

		// somehow the page doesn't load in the locale set in the url, so it can't really be checked if the status was posted
		/*
		if (strpos(strToLower($updateResponse), "your status has been updated") === false) {
			echo("Your Facebook status has not been updated. The login succeed but then there has been a problem for an unknown reason. Try update your status manually and see what happens:<br /><br />".$title."<br /><br /><a href=\"/wp-admin/edit.php\">Go to your posts</a>.");
			// logout from facebook
			getPage("http://m.facebook.com/logout.php");
			deleteFbSessionData();
			exit();
		}
		*/
		
		$fbLogMessage .= "The following status has been published to the profile ".$fbUser.":\n\n".$title."\n\n";

		// if wallId, try post to the wanted wall
		if (isSet($fbWallId) && $fbWallId != NULL && $fbWallId != "") {
			$wallResponse = getPage("http://m.facebook.com/wall.php?id=".$fbWallId."&locale=en_US");
			preg_match('/name="post_form_id" value="(.*?)"/i', $wallResponse, $postFormId);

			if (isSet($postFormId[1])) {
				$postData = "post_form_id=".$postFormId[1]."&comment=".urlencode($title)."&post=Post&locale=en_US";
				$statusFormUrl = "http://m.facebook.com/wall.php?id=".$fbWallId."&locale=en_US";

				// no way to check if this has gone right, FB do not return any message
				$updateResponse = getPage($statusFormUrl, $postData);

				$fbLogMessage .= "It has also been posted on the ".$fbWallId." wall id.\n";
			}
		}

		// if wallId2, try post to the wanted wall
		if (isSet($fbWallId2) && $fbWallId2 != NULL && $fbWallId2 != "") {
			$wallResponse2 = getPage("http://m.facebook.com/wall.php?id=".$fbWallId2."&locale=en_US");
			preg_match('/name="post_form_id" value="(.*?)"/i', $wallResponse2, $postFormId2);

			if (isSet($postFormId2[1])) {
				$postData = "post_form_id=".$postFormId2[1]."&comment=".urlencode($title)."&post=Post&locale=en_US";
				$statusFormUrl2 = "http://m.facebook.com/wall.php?id=".$fbWallId2."&locale=en_US";

				// no way to check if this has gone right, FB do not return any message
				$updateResponse2 = getPage($statusFormUrl2, $postData);

				$fbLogMessage .= "It has also been posted on the ".$fbWallId." wall id.\n";
			}
		}

		// logout from facebook so that a malicious plugin sniffing and sending FB session data won't be able to use it because it's already espired
		getPage("http://m.facebook.com/logout.php");
		// everything has been done, clean the fbSessionData.txt file so that if someone tries to download it, it's empty
		deleteFbSessionData();
	}

	// update the list of sent-to-facebook post ids

	$statusUpdate["fb-post-ids"] .= $post_ID."#";
	update_option('fb-status-update', $statusUpdate);


	$time = getMicrotime() - $startTime;

	$fbLogMessage .= "\nThe whole process took ".number_format($time, 3, ".", "")." seconds, depending on your server performances and bandwidth.\n";
	if ($fbLogEmail != null) {
		sendLogEmail($fbLogEmail, $fbLogMessage);
	}
}

function fbStatusOptionPage(){
	global $fbStatusCookieFile;

	$statusUpdate = get_option('fb-status-update');
	$fbEmail = null;
	$fbPassword = null;
	$fbWallId = null;
	$fbWallId2 = null;
	$fbLogEmail = null;
	$fbPostIds = null;
	$message = false;

	if ($statusUpdate !== false) {
		$fbEmail = $statusUpdate["fb-email"];
		$fbPassword = $statusUpdate["fb-password"];
		$fbWallId = $statusUpdate["fb-wall-id"];
		$fbWallId2 = $statusUpdate["fb-wall-id-2"];
		$fbLogEmail = $statusUpdate["fb-log-email"];
		$fbPostIds = $statusUpdate["fb-post-ids"];
	}

	if (isSet($_POST["action"]) && $_POST["action"] == "fb-status-update") {
		if (isSet($_POST["fb-email"]) && trim($_POST["fb-email"]) != "") {

			$fbEmail = $_POST["fb-email"];

			if (!eregi("^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,4})$", $_POST["fb-email"])) {
				$message = "Email not valid.";
			}

		} else {
			$fbEmail = null;
		}

		if (isSet($_POST["fb-password"]) && trim($_POST["fb-password"]) != "") {
			$fbPassword = $_POST["fb-password"];
		} else {
			$fbPassword = null;
		}

		if (isSet($_POST["fb-wall-id"]) && trim($_POST["fb-wall-id"]) != "") {
			$fbWallId = $_POST["fb-wall-id"];
		} else {
			$fbWallId = null;
		}

		if (isSet($_POST["fb-wall-id-2"]) && trim($_POST["fb-wall-id-2"]) != "") {
			$fbWallId2 = $_POST["fb-wall-id-2"];
		} else {
			$fbWallId2 = null;
		}

		if (isSet($_POST["fb-log-email"]) && trim($_POST["fb-log-email"]) != "") {

			$fbLogEmail = $_POST["fb-log-email"];

			if (!eregi("^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,4})$", $_POST["fb-log-email"])) {
				$message = "Log Email not valid.";
			}

		} else {
			$fbLogEmail = null;
		}

		if ($fbEmail == null || $fbPassword == null) {
			$message = "Email and Password fields are mandatory.";
		}

		if ($message == false) {
			// no validation errors here

			if ($statusUpdate == false) {
				// store for the very first time
				$statusUpdate = array();
				$statusUpdate["fb-email"] = $fbEmail;
				$statusUpdate["fb-password"] = $fbPassword;
				$statusUpdate["fb-wall-id"] = $fbWallId;
				$statusUpdate["fb-wall-id-2"] = $fbWallId2;
				$statusUpdate["fb-log-email"] = $fbLogEmail;
				$statusUpdate["fb-post-ids"] = $fbPostIds;
				add_option('fb-status-update', $statusUpdate, ' ', 'no');
				$message = "Plugin settings have been saved.";
			} else {
				// update
				$statusUpdate = array();
				$statusUpdate["fb-email"] = $fbEmail;
				$statusUpdate["fb-password"] = $fbPassword;
				$statusUpdate["fb-wall-id"] = $fbWallId;
				$statusUpdate["fb-wall-id-2"] = $fbWallId2;
				$statusUpdate["fb-log-email"] = $fbLogEmail;
				$statusUpdate["fb-post-ids"] = $fbPostIds;
				update_option('fb-status-update', $statusUpdate);
				$message = true;
				$message = "Plugin settings have been updated.";
			}
		}

	}
?>
<div class="wrap">
	<h2>Facebook Status Updater</h2>

	<form method="post">
		<div id="poststuff" class="metabox-holder">
			<div class="postbox">
				<h3 class="hndle"><span>Facebook Status Updater</h3>
				<div class="inside">
					<p>This plugin will update the status of a certains Facebook Profile with the post title and link, every time a new post is published. Additionally, the plugin might add the same message on a wall of the given page, if the Facebook user is also admin of that page.</p>
					<p>In order to do that, you should provide your Facebook login data into the form below. Please note that your Facebook login information are stored in your blog database on the server your blog is hosted. If you think your blog is not a safe place to store that data, don't use this plugin.</p>
					<p>If your Facebook login data is not correct, nothing will happen.</p>
					<?php if (!is_writable($fbStatusCookieFile)) { ?>
						<p style="color:#990000">Php/Wordpress cannot write into this file: <strong><?php echo($fbStatusCookieFile) ?></strong>. Please ensure PHP has the correct permissions set to write and update that file. If you don't know what I'm speaking about, please contact your server admin / webmaster. If you don't want to see this message every time you publish a new post while you try solving the problem, just <a href="/wp-admin/plugins.php">disable this plugin</a>. <a href="http://codex.wordpress.org/Changing_File_Permissions">More about file permissions on Wordpress</a></p>
					<? } ?>
					<p>* = required field</p>
					
					<?php if ($message != false) { ?>
						<div id="message" class="updated fade"><p><?php echo($message); ?></p></div>
					<?php } ?>
					
					<table class="form-table">
						<tr valign="top">
							<td><label for="fb-email"><strong>Facebook email*</strong></label></td>
							<td><input style="width: 250px;" id="fb-email" name="fb-email" type="text" value="<?php echo($fbEmail); ?>" /></td>
							<td></td>
						</tr>
						<tr valign="top">
							<td><label for="fb-password"><strong>Facebook password*</strong></label></td>
							<td><input style="width: 250px;" id="fb-password" name="fb-password" type="password" value="<?php echo($fbPassword); ?>" /></td>
							<td></td>
						</tr>
						<tr valign="top">
							<td><label for="fb-wall-id"><strong>Facebook wall id</strong></label></td>
							<td><input style="width: 250px;" id="fb-wall-id" name="fb-wall-id" type="text" value="<?php echo($fbWallId); ?>" /></td>
							<td>Optional wall id where to post your new article. Where to find a Facebook Page wall id? Example:<br />http://www.facebook.com/pages/YourPageName/<strong>1234567890</strong>?ref=ts<br /><strong>If you fill this field with a Page Id that the provided email is not admin of, the plugin won't do anything and Facebook will ban you. Understand?</strong></td>
						</tr>
						<tr valign="top">
							<td><label for="fb-wall-id-2"><strong>Facebook wall id 2</strong></label></td>
							<td><input style="width: 250px;" id="fb-wall-id-2" name="fb-wall-id-2" type="text" value="<?php echo($fbWallId2); ?>" /></td>
							<td>Additional optional wall id where to post your new article. Where to find a Facebook Page wall id? Example:<br />http://www.facebook.com/pages/YourPageName/<strong>1234567890</strong>?ref=ts<br /><strong>If you fill this field with a Page Id that the provided email is not admin of, the plugin won't do anything and Facebook will ban you. Understand?</strong></td>
						</tr>
						<tr valign="top">
							<td><label for="fb-log-email"><strong>Log email</strong></label></td>
							<td><input style="width: 250px;" id="fb-log-email" name="fb-log-email" type="text" value="<?php echo($fbLogEmail); ?>" /></td>
							<td>The Facebook update during the publishing process happens silently. If you want to know what happened, just inser your personal email in this field and you'll receive the log.</td>
						</tr>
					</table>
				</div>
			</div>

			<input type="hidden" name="action" value="fb-status-update" />
			<p class="submit"><input type="submit" name="Submit" value="Salva" /></p>

			<?php if ($fbPostIds != null) { ?>

				<div class="postbox">
					<h3 class="hndle"><span>Sent posts log</h3>
					<div class="inside">
						<p>Here is what you have sent to Facebook:</p>
						<p>
						<?php
							$fbPostIds = explode("#", $fbPostIds);
							foreach($fbPostIds as $fbPostId) {
								if (trim($fbPostId) != "") {
						?>
									<a href="/?p=<?php echo($fbPostId); ?>" target="_blank"><?php echo($fbPostId); ?></a> 
						<?php
								}
							}	
						?>
						</p>
					</div>
				</div>
			<?php } ?>
		</div>
	</form>
</div>
<?php
}

function fbStatusAct() {
	// clean everything @ activation
	//delete_option('fb-status-update');
	
	if (!function_exists("curl_init")) {
		deactivate_plugins(__FILE__);
		die("This plugin needs <a href=\"http://www.php.net/curl\"></a> to be installed on your server in order to run properly.");
	}
}

add_action("publish_post", "fbStatusUpdater", 0);

function addFbStatusUpdaterOptionPage() {
	add_options_page('Facebook Status Updater', 'Facebook Status Updater', 9, basename(__FILE__), "fbStatusOptionPage");
}

add_action('admin_menu', 'addFbStatusUpdaterOptionPage');

register_activation_hook( __FILE__, "fbStatusAct");

?>